@@ -20,6 +20,7 @@
 #include "bonobo-item-handler.h"
 
 #define PARENT_TYPE BONOBO_TYPE_OBJECT
+
 static GObjectClass *bonobo_item_handler_parent_class;
 
 struct _BonoboItemHandlerPrivate
@@ -58,7 +59,7 @@ bonobo_marshal_POINTER__DUMMY_BOXED (GClosure     *closure,
 	callback = (GMarshalFunc_POINTER__POINTER_BOXED) (marshal_data ? marshal_data : cc->callback);
 
 	v_return = callback (data1,
-			     NULL,
+			     data2,
 			     g_value_get_boxed (param_values + 1),
 			     data2);
 
@@ -99,7 +100,7 @@ bonobo_marshal_BOXED__STRING_BOOLEAN_DUMMY_BOXED (GClosure     *closure,
 	v_return = callback (data1,
 			     (char*) g_value_get_string (param_values + 1),
 			     g_value_get_boolean (param_values + 2),
-			     NULL,
+			     data2,
 			     g_value_get_boxed (param_values + 3),
 			     data2);
 
@@ -120,10 +121,10 @@ impl_enum_objects (PortableServer_Servant servant, CORBA_Environment *ev)
 		Bonobo_ItemContainer_ObjectNames *ret;
 
 		bonobo_closure_invoke (handler->priv->enum_objects,
-				       G_TYPE_POINTER,              &ret,
-				       BONOBO_TYPE_ITEM_HANDLER,    handler,
-				       BONOBO_TYPE_CORBA_EXCEPTION, ev,
-				       0);
+				       G_TYPE_POINTER,                    &ret,
+				       BONOBO_TYPE_ITEM_HANDLER,           handler,
+				       BONOBO_TYPE_STATIC_CORBA_EXCEPTION, ev,
+				       G_TYPE_INVALID);
 
 		return ret;
 	} else
@@ -139,17 +140,16 @@ impl_get_object (PortableServer_Servant servant,
 	BonoboObject *object = bonobo_object_from_servant (servant);
 	BonoboItemHandler *handler = BONOBO_ITEM_HANDLER (object);
 
-	if (handler->priv->get_object)
-	{
+	if (handler->priv->get_object) {
 		Bonobo_Unknown ret;
 
 		bonobo_closure_invoke (handler->priv->get_object,
-				       BONOBO_TYPE_UNKNOWN,         &ret,
-				       BONOBO_TYPE_ITEM_HANDLER,    handler,
-				       G_TYPE_STRING,               item_name,
-				       G_TYPE_BOOLEAN,              only_if_exists,
-				       BONOBO_TYPE_CORBA_EXCEPTION, ev,
-				       0);
+				       BONOBO_TYPE_STATIC_UNKNOWN,         &ret,
+				       BONOBO_TYPE_ITEM_HANDLER,           handler,
+				       G_TYPE_STRING,                      item_name,
+				       G_TYPE_BOOLEAN,                     only_if_exists,
+				       BONOBO_TYPE_STATIC_CORBA_EXCEPTION, ev,
+				       G_TYPE_INVALID);
 				       
 		return ret;
 	} else
@@ -161,13 +161,15 @@ bonobo_item_handler_finalize (GObject *object)
 {
 	BonoboItemHandler *handler = BONOBO_ITEM_HANDLER (object);
 
-	if (handler->priv)
-	{
-		g_closure_unref (handler->priv->enum_objects);
-		g_closure_unref (handler->priv->get_object);
+	if (handler->priv) {
+		if (handler->priv->enum_objects)
+			g_closure_unref (handler->priv->enum_objects);
+
+		if (handler->priv->get_object)
+			g_closure_unref (handler->priv->get_object);
 
 		g_free (handler->priv);
-		handler->priv = 0;
+		handler->priv = NULL;
 	}
 
 	bonobo_item_handler_parent_class->finalize (object);
@@ -198,13 +200,13 @@ bonobo_item_handler_class_init (BonoboItemHandlerClass *klass)
 }
 
 BONOBO_TYPE_FUNC_FULL (BonoboItemHandler, 
-			   Bonobo_ItemContainer,
-			   PARENT_TYPE,
-			   bonobo_item_handler);
+		       Bonobo_ItemContainer,
+		       PARENT_TYPE,
+		       bonobo_item_handler)
 
 /**
  * bonobo_item_handler_construct:
- * @container: The handler object to construct
+ * @handler: The handler object to construct
  * @enum_objects: The closure implementing enumObjects
  * @get_object: The closure implementing getObject
  *
@@ -220,17 +222,22 @@ bonobo_item_handler_construct (BonoboItemHandler *handler,
 {
 	g_return_val_if_fail (handler != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_ITEM_HANDLER (handler), NULL);
-	
-	handler->priv->enum_objects = bonobo_closure_store
-		(enum_objects, bonobo_marshal_POINTER__DUMMY_BOXED);
-	handler->priv->get_object   = bonobo_closure_store
-		(get_object, bonobo_marshal_BOXED__STRING_BOOLEAN_DUMMY_BOXED);
+
+	if (enum_objects)
+		handler->priv->enum_objects = bonobo_closure_store
+			(enum_objects, bonobo_marshal_POINTER__DUMMY_BOXED);
+	if (get_object)
+		handler->priv->get_object = bonobo_closure_store
+			(get_object, bonobo_marshal_BOXED__STRING_BOOLEAN_DUMMY_BOXED);
 	
 	return handler;
 }
 
 /**
  * bonobo_item_handler_new:
+ * @enum_objects: callback invoked for Bonobo::ItemContainer::enum_objects
+ * @get_object: callback invoked for Bonobo::ItemContainer::get_objects
+ * @user_data: extra data passed on the callbacks
  *
  * Creates a new BonoboItemHandler object.  These are used to hold
  * client sites.
@@ -242,13 +249,24 @@ bonobo_item_handler_new (BonoboItemHandlerEnumObjectsFn enum_objects,
 			 BonoboItemHandlerGetObjectFn   get_object,
 			 gpointer                       user_data)
 {
-	return bonobo_item_handler_new_closure (
-		g_cclosure_new (G_CALLBACK (enum_objects), user_data, NULL),
-		g_cclosure_new (G_CALLBACK (get_object), user_data, NULL));
+	GClosure *enum_objects_closure = NULL;
+	GClosure *get_object_closure = NULL;
+
+	if (enum_objects)
+		enum_objects_closure =
+			g_cclosure_new (G_CALLBACK (enum_objects), user_data, NULL);
+
+	if (get_object)
+		get_object_closure =
+			g_cclosure_new (G_CALLBACK (get_object), user_data, NULL);
+
+	return bonobo_item_handler_new_closure (enum_objects_closure, get_object_closure);
 }
 
 /**
  * bonobo_item_handler_new_closure:
+ * @enum_objects: closure invoked for Bonobo::ItemContainer::enum_objects
+ * @get_object: closure invoked for Bonobo::ItemContainer::get_objects
  *
  * Creates a new BonoboItemHandler object.  These are used to hold
  * client sites.
@@ -266,6 +284,28 @@ bonobo_item_handler_new_closure (GClosure *enum_objects,
 	return bonobo_item_handler_construct (handler, enum_objects, get_object);
 }
 
+static GSList *
+bonobo_item_option_new_append (GSList  *option_list,
+			       GString *key,
+			       GString *value)
+{
+	BonoboItemOption *option;
+
+	g_assert (key && key->str);
+
+	option = g_new0 (BonoboItemOption, 1);
+
+	option->key  = key->str;
+	g_string_free (key, FALSE);
+
+	if (value) {
+		option->value = value->str;
+		g_string_free (value, FALSE);
+	}
+
+	return g_slist_append (option_list, option);
+}
+
 /**
  * bonobo_parse_item_options:
  * @option_string: a string with a list of options
@@ -292,61 +332,54 @@ bonobo_item_handler_new_closure (GClosure *enum_objects,
 GSList *
 bonobo_item_option_parse (const char *option_string)
 {
-	GSList *list = NULL;
-	GString *key = NULL;
-	BonoboItemOption *option = NULL;
+	GSList     *list  = NULL;
+	GString    *key   = NULL;
+	GString    *value = NULL;
 	const char *p;
 	
-	for (p = option_string; *p; p++) {
-		if (*p == '=' ) {
-			GString *value = NULL;
+	for (p = option_string; *p; p++)
+		switch (*p) {
+		case '=':
+			if (!key || value)
+				goto parse_error_free;
+
+			value = g_string_new (NULL);
+			break;
+		case ';':
 			if (!key)
-				return list;
-			
-			option = g_new0 (BonoboItemOption, 1);
-			option->key = key->str;
-			g_string_free (key, FALSE);
-			key = NULL;
-
-			for (p++; *p; p++) {
-				if (*p == ';')
-					goto next;
-				if (!value)
-					value = g_string_new ("");
-
-				if (*p == '\\') {
-					p++;
-					if (*p == 0)
-						break;
-					g_string_append_c (value, *p);
-					continue;
-				}
-				g_string_append_c (value, *p);
-			}
-		next:
-			if (value) {
-				option->value = value->str;
-				g_string_free (value, FALSE);
-			}
-			list = g_slist_append (list, option);
-			if (*p == 0)
 				break;
-		} else {
-			if (key == NULL)
-				key = g_string_new ("");
-			g_string_append_c (key, *p);
-		}
-	}
 
-	if (key) {
-		BonoboItemOption *option = g_new (BonoboItemOption, 1);
-		
-		option->key = key->str;
-		g_string_free (key, FALSE);
-		
-		list = g_slist_append (list, option);
-	}
-	
+			list = bonobo_item_option_new_append (list, key, value);
+			key = NULL; value = NULL;
+			break;
+		case '\\':
+			if (!key || !*++p)
+				goto parse_error_free;
+
+			/* drop through */
+		default:
+			if (!key)
+				key = g_string_new (NULL);
+
+			if (value)
+				g_string_append_c (value, *p);
+			else
+				g_string_append_c (key, *p);
+			break;
+		}
+
+	if (key)
+		list = bonobo_item_option_new_append (list, key, value);
+
+	return list;
+
+ parse_error_free:
+	if (key)
+		g_string_free (key, TRUE);
+
+	if (value)
+		g_string_free (value, TRUE);
+
 	return list;
 }
 
@@ -367,6 +400,8 @@ bonobo_item_options_free (GSList *options)
 		g_free (option->key);
 		if (option->value)
 			g_free (option->value);
+
+		g_free (option);
 	}
 
 	g_slist_free (options);
